from zope.interface import Interface

class IStripeModeChooser(Interface):
    """ Interface for content types which can control 
        the Stripe mode, overriding the global control 
        panel setting 
    """

    def get_stripe_mode():
        """ Returns either 'live' or 'test' """

class IStripeEnabledView(Interface):
    """ Interface for views using Stripe """

    def show_stripe():
        """ Returns boolean for whether or not to include the stripe javascript """


# Webhook triggered events

class BaseWebhookEvent(object):
    def __init__(self, data):
        self.data = data
# Account events

class IAccountUpdatedEvent(BaseWebhookEvent):
    """ describes an account - Occurs whenever an account status or property has changed. """

class IAccountApplicationDeauthorizedEvent(BaseWebhookEvent):
    """ describes an application - Occurs whenever a user deauthorizes an application. Sent to the related application only. """

# Balance events

class IBalanceAvailableEvent(BaseWebhookEvent):
    """ describes a balance - Occurs whenever your Stripe balance has been updated (e.g. when a charge collected is available to be paid out). By default, Stripe will automatically transfer any funds in your balance to your bank account on a daily basis. """

# Charge events

class IChargeSucceededEvent(BaseWebhookEvent):
    """ describes a charge - Occurs whenever a new charge is created and is successful. """

class IChargeFailedEvent(BaseWebhookEvent):
    """ describes a charge - Occurs whenever a failed charge attempt occurs. """

class IChargeRefundedEvent(BaseWebhookEvent):
    """ describes a charge - Occurs whenever a charge is refunded, including partial refunds. """

class IChargeCapturedEvent(BaseWebhookEvent):
    """ describes a charge - Occurs whenever a previously uncaptured charge is captured. """

class IChargeDisputeCreatedEvent(BaseWebhookEvent):
    """ describes a dispute - Occurs whenever a customer disputes a charge with their bank (chargeback). """

class IChargeDisputeUpdatedEvent(BaseWebhookEvent):
    """ describes a dispute - Occurs when the dispute is updated (usually with evidence). """

class IChargeDisputeClosedEvent(BaseWebhookEvent):    """ describes a dispute - Occurs when the dispute is resolved and the dispute status changes to won or lost. """# Customer events

class ICustomerCreatedEvent(BaseWebhookEvent):
    """ describes a customer - Occurs whenever a new customer is created. """

class ICustomerUpdatedEvent(BaseWebhookEvent):
    """ describes a customer - Occurs whenever any property of a customer changes. """

class ICustomerDeletedEvent(BaseWebhookEvent):
    """ describes a customer - Occurs whenever a customer is deleted. """

class ICustomerCardCreatedEvent(BaseWebhookEvent):
    """ describes a card - Occurs whenever a new card is created for the customer. """

class ICustomerCardUpdatedEvent(BaseWebhookEvent):
    """ describes a card - Occurs whenever a card's details are changed. """

class ICustomerCardDeletedEvent(BaseWebhookEvent):
    """ describes a card - Occurs whenever a card is removed from a customer. """

class ICustomerSubscriptionCreatedEvent(BaseWebhookEvent):
    """ describes a subscription - Occurs whenever a customer with no subscription is signed up for a plan. """

class ICustomerSubscriptionCreatedEvent(BaseWebhookEvent):
    """ describes a subscription - Occurs whenever a customer with no subscription is signed up for a plan. """

class ICustomerSubscriptionUpdatedEvent(BaseWebhookEvent):
    """ describes a subscription - Occurs whenever a subscription changes. Examples would include switching from one plan to another, or switching status from trial to active. """

class ICustomerSubscriptionDeletedEvent(BaseWebhookEvent):
    """ describes a subscription - Occurs whenever a customer ends their subscription. """

class ICustomerSubscriptionTrialWillEndEvent(BaseWebhookEvent):
    """ describes a subscription - Occurs three days before the trial period of a subscription is scheduled to end. """

class ICustomerDiscountCreatedEvent(BaseWebhookEvent):
    """ describes a discount - Occurs whenever a coupon is attached to a customer. """

class ICustomerDiscountUpdatedEvent(BaseWebhookEvent):
    """ describes a discount - Occurs whenever a customer is switched from one coupon to another. """

class ICustomerDiscountDeletedEvent(BaseWebhookEvent):
    """ describes a discount - Occurs whenever a customer's discount is removed. """

# Invoice events

class IInvoiceCreatedEvent(BaseWebhookEvent):
    """ describes an invoice - Occurs whenever a new invoice is created. If you are using webhooks, Stripe will wait one hour after they have all succeeded to attempt to pay the invoice; the only exception here is on the first invoice, which gets created and paid immediately when you subscribe a customer to a plan. If your webhooks do not all respond successfully, Stripe will continue retrying the webhooks every hour and will not attempt to pay the invoice. After 3 days, Stripe will attempt to pay the invoice regardless of whether or not your webhooks have succeeded. See how to respond to a webhook. """

class IInvoiceUpdatedEvent(BaseWebhookEvent):
    """ describes an invoice - Occurs whenever an invoice changes (for example, the amount could change). """

class IInvoicePaymentSucceededEvent(BaseWebhookEvent):
    """ describes an invoice - Occurs whenever an invoice attempts to be paid, and the payment succeeds. """

class IInvoicePaymentFailedEvent(BaseWebhookEvent):
    """ describes an invoice - Occurs whenever an invoice attempts to be paid, and the payment fails. """

class IInvoiceItemCreatedEvent(BaseWebhookEvent):
    """ describes an invoice item - Occurs whenever an invoice item is created. """

class IInvoiceItemUpdatedEvent(BaseWebhookEvent):
    """ describes an invoice item - Occurs whenever an invoice item is updated. """

class IInvoiceItemDeletedEvent(BaseWebhookEvent):
    """ describes an invoice item - Occurs whenever an invoice item is deleted. """

# Plan events

class IPlanCreatedEvent(BaseWebhookEvent):
    """ describes a plan - Occurs whenever a plan is created. """

class IPlanUpdatedEvent(BaseWebhookEvent):
    """ describes a plan - Occurs whenever a plan is updated. """

class IPlanDeletedEvent(BaseWebhookEvent):
    """ describes a plan - Occurs whenever a plan is deleted. """

# Coupon related events

class ICouponCreatedEvent(BaseWebhookEvent):
    """ describes a coupon - Occurs whenever a coupon is created. """

class ICouponDeletedEvent(BaseWebhookEvent):
    """ describes a coupon - Occurs whenever a coupon is deleted. """

# Transfer events

class ITransferCreatedEvent(BaseWebhookEvent):
    """ describes a transfer - Occurs whenever a new transfer is created. """

class ITransferUpdatedEvent(BaseWebhookEvent):
    """ describes a transfer - Occurs whenever the amount of a pending transfer is updated. """

class ITransferPaidEvent(BaseWebhookEvent):
    """ describes a transfer - Occurs whenever a sent transfer is expected to be available in the destination bank account. If the transfer failed, a transfer.failed webhook will additionally be sent at a later time. """

class ITransferFailedEvent(BaseWebhookEvent):
    """ describes a transfer - Occurs whenever Stripe attempts to send a transfer and that transfer fails. """

# Ping event

class IPingEvent(BaseWebhookEvent):
    """ has no description - May be sent by Stripe at any time to see if a provided webhook URL is working. """



class DummyStripeUtility(object):
    def get_stripe_api(self):
        return DummyStripe()
 
class DummyStripe(object):
    api_key = '123456789'

    @property
    def Event(self):
        return DummyEvents()


class DummyEvents(object):
    def __init__(self):
        self.dummy_event = {
            "id": "test_passing",
            "created": 123456789,
            "livemode": False,
            "type": "charge.succeeded",
            "data": {
                "object": {
                    "id": "ch_2OmwsdSNjmFvGQ",
                    "object": "charge",
                    "created": 1376706172,
                    "livemode": False,
                    "paid": True,
                    "amount": 500,
                    "currency": "usd",
                    "refunded": False,
                    "card": {
                        "id": "card_2OmwN1y86pCJy2",
                        "object": "card",
                        "last4": "4242",
                        "type": "Visa",
                        "exp_month": 1,
                        "exp_year": 2050,
                        "fingerprint": "qhjxpr7DiCdFYTlH",
                        "customer": None,
                        "country": "US",
                        "name": None,
                        "address_line1": None,
                        "address_line2": None,
                        "address_city": None,
                        "address_state": None,
                        "address_zip": None,
                        "address_country": None,
                        "cvc_check": "pass",
                        "address_line1_check": None,
                        "address_zip_check": None
                    },
                    "captured": True,
                    "refunds": [
                    ],
                    "balance_transaction": "txn_2OmwHoZeThcMdu",
                    "failure_message": None,
                    "failure_code": None,
                    "amount_refunded": 0,
                    "customer": None,
                    "invoice": None,
                    "description": None,
                    "dispute": None,
                }
            },
            "object": "event",
            "pending_webhooks": 0,
            "request": "iar_2Omw24HYWicSuq"
        } 

    def retrieve(self, id):
        self.dummy_event['id'] = id
        if id == 'test_failing_id':
            self.dummy_event['id'] = 'test_id_failing'
        if id == 'test_failing_type':
            self.dummy_event['type'] = 'failing'
        if id == 'test_failing_created':
            self.dummy_event['created'] = 0 
        return self.dummy_event



import stripe
from five import grok
from zope.interface import Interface
from zope.site.hooks import getSite
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from collective.stripe.controlpanel import IStripeSettings
from collective.stripe.interfaces import IStripeModeChooser

def get_settings():
    registry = getUtility(IRegistry)
    return registry.forInterface(IStripeSettings, False)

class IStripeUtility(Interface):
    """ A global utility providing methods to access the Stripe API """

    def get_stripe_api():
        """ returns the stripe api module with the api_key set from the control panel """

    def charge_card(token, amount, description, **kwargs):
        """ charges a card (represented by a Stripe.js card token) """

    def charge_customer(customer_id, amount, description, **kwargs):
        """ charges a customer looked up by customer_id.  customer must already exist """

    def create_customer(token, description, **kwargs):
        """ creates a customer using a card token from Stripe.js """

    def subscribe_customer(customer_id, plan, quantity, **kwargs):
        """ subscribe and existing customer to a plan.  if subscription
            already exists for the plan, the subscription is updated """

class StripeUtility(object):
    grok.implements(IStripeUtility)

    def get_stripe_api(self, context=None, mode=None):
        active_mode = 'live'
        settings = get_settings()

        if mode is not None:
            active_mode = mode
        else:
            active_mode = 'live'
            if settings.mode is not None:
                active_mode == settings.mode

            if active_mode != 'test' and context is not None:
                active_mode = self.get_mode_for_context(context)

        stripe.api_key = getattr(settings, '%s_secret_key' % active_mode)
        return stripe

    def get_mode_for_context(self, context):
        if IStripeModeChooser.providedBy(context):
            return context.get_stripe_mode()
        return get_settings().mode

    def charge_card(self, token, amount, description, context=None, **kwargs):
        settings = get_settings()
        stripe = self.get_stripe_api(context=context)

        res = stripe.Charge.create(
            amount = amount,
            currency = settings.currency,
            card = token,
            description = description,
            **kwargs
        )
        return res

    def charge_customer(self, customer_id, amount, description, context=None, **kwargs):
        settings = get_settings()
        stripe = self.get_stripe_api(context=context)

        res = stripe.Charge.create(
            amount = amount,
            currency = settings.currency,
            card = token,
            description = description,
            **kwargs
        )
        return res

    def create_customer(self, token, description, context=None, **kwargs):
        settings = get_settings()
        stripe = self.get_stripe_api(context=context)

        res = stripe.Customer.create(
            card = token,
            description = description,
            **kwargs
        )
        return res

    def subscribe_customer(self, customer_id, plan, quantity, context=None, **kwargs):
        settings = get_settings()
        stripe = self.get_stripe_api(context=context)

        cu = stripe.Customer.retrieve(customer_id)
        res = cu.update_subscription(
            plan=plan, 
            quantity=quantity,
            **kwargs
        )
        return res

grok.global_utility(StripeUtility, provides=IStripeUtility)



from five import grok
from zope.component import getUtility
from zope import schema
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary

from collective.stripe.utils import IStripeUtility


class StripePlansVocabulary(object):
    grok.implements(IVocabularyFactory)

    def __call__(self, context):
        return self.get_plans(context)

    def get_plans(self, context):
        api = getUtility(IStripeUtility).get_stripe_api(context)
        terms = []

        for info in api.Plan.all()['data']:
            name = '%s (per %s)' % (info['name'], info['interval'])
            terms.append(SimpleVocabulary.createTerm(info['id'], info['id'], name))

        return SimpleVocabulary(terms)

grok.global_utility(StripePlansVocabulary, name=u"collective.stripe.plans")



from zope.component import adapter, getMultiAdapter
from zope.interface import implementer, implements, implementsOnly

from z3c.form.interfaces import IField, IFieldWidget, IFormLayer, NOVALUE
from z3c.form.widget import FieldWidget
from z3c.form.browser import text

from Products.Five.browser import BrowserView
from zope.publisher.interfaces import IPublishTraverse, NotFound

class StripeTokenWidget(text.TextWidget):
    """A widget for a named file object
    """
    implementsOnly(IFieldWidget)

    klass = u'stripe-token-widget'
    value = None # don't default to a string
    
    def extract(self, default=NOVALUE):
        value = super(StripeTokenWidget, self).extract(default)
        return value

@implementer(IFieldWidget)
@adapter(IField, IFormLayer)
def StripeTokenWidget(field, request):
    return FieldWidget(field, StripeTokenWidget(request))



from plone.testing import z2
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting
from plone.app.testing import ploneSite
from plone.app.testing import quickInstallProduct
import transaction


class CollectiveStripeLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import collective.stripe
        self.loadZCML(package=collective.stripe)

        z2.installProduct(app, 'collective.stripe')

    def setUpPloneSite(self, portal):
        quickInstallProduct(portal, 'collective.stripe')
        self.applyProfile(portal, 'collective.stripe:default')
        transaction.commit()

    def tearDownZope(self, app):
        z2.uninstallProduct(app, 'collective.stripe')


FIXTURE = CollectiveStripeLayer()
INTEGRATION_TESTING = IntegrationTesting(bases=(FIXTURE,), name='collective.stripe:Integration')
FUNCTIONAL_TESTING = FunctionalTesting(bases=(FIXTURE,), name='collective.stripe:Functional')



from zope.interface import Interface
from five import grok
from Acquisition import aq_inner
from Products.CMFCore.utils import getToolByName
from plone.app.layout.viewlets.interfaces import IHtmlHead
from collective.stripe.interfaces import IStripeModeChooser
from collective.stripe.interfaces import IStripeEnabledView
from collective.stripe.utils import get_settings

STRIPE_JS_HTML = """
    <script type="text/javascript" src="https://js.stripe.com/v1/"></script>
    <script type="text/javascript">
        (function ($) {
            $(document).ready(function() {
                Stripe.setPublishableKey('%s');
            });
        })(jQuery);
    </script>
"""

class StripeJs(grok.Viewlet):
    """ Inject the Stripe.js script tag and configuration if relevant"""
    grok.name('collective.stripe.stripejs')
    grok.require('zope2.View')
    grok.context(Interface)
    grok.viewletmanager(IHtmlHead)

    def render(self):
        # Only run on views marked with IStripeEnabledView
        if not IStripeEnabledView.providedBy(self.view):
            return ""

        key = self.get_key()
        return STRIPE_JS_HTML % key

    def get_key(self):
        settings = get_settings()

        if IStripeModeChooser.providedBy(self.context):
            mode = self.context.get_stripe_mode()
        else:
            mode = settings.mode

        if mode == 'live':
            return settings.live_publishable_key

        if mode == 'test':
            return settings.test_publishable_key



from setuptools import setup, find_packages
import os

version = '0.3.2'

setup(name='collective.stripe',
      version=version,
      description="Stripe Credit Card Processing Integration",
      long_description=open("README.md"),
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='',
      author_email='',
      url='http://github/collective/collective.stripe',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'stripe',
          'five.grok',
          # -*- Extra requirements: -*-
      ],
      extras_require = {
          'test': [
              'plone.app.testing',
          ]
      },
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,

      )



from five import grok

from z3c.form import interfaces

from zope import schema
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from zope.schema.interfaces import IVocabularyFactory
from zope.interface import Interface

from plone.app.registry.browser import controlpanel

MODE_VOCABULARY = SimpleVocabulary((
    SimpleTerm(value="live", title=u"Live (Production Mode)"),
    SimpleTerm(value="test", title=u"Test (Test Mode, no transactions really go through, be careful)"),
))
class StripeModesVocabulary(object):
    grok.implements(IVocabularyFactory)
    def __call__(self, context):
        return MODE_VOCABULARY
grok.global_utility(StripeModesVocabulary, name=u"collective.stripe.modes")

CURRENCY_VOCABULARY = SimpleVocabulary((
    SimpleTerm(value="usd", title=u"USD - $"),
))

class IStripeSettings(Interface):
    """ Global settings for collective.stripe stored in the registry """
    mode = schema.Choice(
        title=u"Mode",
        description=u"Determines whether calls to the Stripe API are made using the Test or the Live key",
        vocabulary=MODE_VOCABULARY,
        default=u"test",
    )
    test_secret_key = schema.TextLine(
        title=u"Test Secret Key",
        description=u"Located under Account Settings -> API Keys when logged into stripe.com",
    )
    test_publishable_key = schema.TextLine(
        title=u"Test Publishable Key",
        description=u"Located under Account Settings -> API Keys when logged into stripe.com",
    )
    live_secret_key = schema.TextLine(
        title=u"Live Secret Key",
        description=u"Located under Account Settings -> API Keys when logged into stripe.com",
    )
    live_publishable_key = schema.TextLine(
        title=u"Live Publishable Key",
        description=u"Located under Account Settings -> API Keys when logged into stripe.com",
    )
    currency = schema.Choice(
        title=u"Currency",
        description=u"The currency, used in calls to the Stripe API",
        vocabulary=CURRENCY_VOCABULARY,
        default=u"usd",
    )

class StripeSettingsEditForm(controlpanel.RegistryEditForm):
    schema = IStripeSettings
    label = u"Stripe Settings"
    description = u"Configuration for collective.stripe providing integration with Stripe for payment processing"

    def updateFields(self):
        super(StripeSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(StripeSettingsEditForm, self).updateWidgets()

class StripeSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = StripeSettingsEditForm



import json
from five import grok
from zope.component import getUtility
from zope.event import notify
from Products.CMFPlone.interfaces import IPloneSiteRoot
from collective.stripe.utils import IStripeUtility

from collective.stripe.interfaces import \
    IAccountUpdatedEvent, \
    IAccountApplicationDeauthorizedEvent, \
    IBalanceAvailableEvent, \
    IChargeSucceededEvent, \
    IChargeFailedEvent, \
    IChargeRefundedEvent, \
    IChargeCapturedEvent, \
    IChargeDisputeCreatedEvent, \
    IChargeDisputeUpdatedEvent, \
    IChargeDisputeClosedEvent, \
    ICustomerCreatedEvent, \
    ICustomerUpdatedEvent, \
    ICustomerDeletedEvent, \
    ICustomerCardCreatedEvent, \
    ICustomerCardUpdatedEvent, \
    ICustomerCardDeletedEvent, \
    ICustomerSubscriptionCreatedEvent, \
    ICustomerSubscriptionCreatedEvent, \
    ICustomerSubscriptionUpdatedEvent, \
    ICustomerSubscriptionDeletedEvent, \
    ICustomerSubscriptionTrialWillEndEvent, \
    ICustomerDiscountCreatedEvent, \
    ICustomerDiscountUpdatedEvent, \
    ICustomerDiscountDeletedEvent, \
    IInvoiceCreatedEvent, \
    IInvoiceUpdatedEvent, \
    IInvoicePaymentSucceededEvent, \
    IInvoicePaymentFailedEvent, \
    IInvoiceItemCreatedEvent, \
    IInvoiceItemUpdatedEvent, \
    IInvoiceItemDeletedEvent, \
    IPlanCreatedEvent, \
    IPlanUpdatedEvent, \
    IPlanDeletedEvent, \
    ICouponCreatedEvent, \
    ICouponDeletedEvent, \
    ITransferCreatedEvent, \
    ITransferUpdatedEvent, \
    ITransferPaidEvent, \
    ITransferFailedEvent, \
    IPingEvent


EVENTS_MAP = {
    'account.updated': IAccountUpdatedEvent,
    'account.application.deauthorized': IAccountApplicationDeauthorizedEvent,
    'balance.available': IBalanceAvailableEvent,
    'charge.succeeded': IChargeSucceededEvent,
    'charge.failed': IChargeFailedEvent,
    'charge.refunded': IChargeRefundedEvent,
    'charge.captured': IChargeCapturedEvent,
    'charge.dispute.created': IChargeDisputeCreatedEvent,
    'charge.dispute.updated': IChargeDisputeUpdatedEvent,
    'charge.dispute.closed': IChargeDisputeClosedEvent,
    'customer.created': ICustomerCreatedEvent,
    'customer.updated': ICustomerUpdatedEvent,
    'customer.deleted': ICustomerDeletedEvent,
    'customer.card.created': ICustomerCardCreatedEvent,
    'customer.card.updated': ICustomerCardUpdatedEvent,
    'customer.card.deleted': ICustomerCardDeletedEvent,
    'customer.subscription.created': ICustomerSubscriptionCreatedEvent,
    'customer.subscription.updated': ICustomerSubscriptionUpdatedEvent,
    'customer.subscription.deleted': ICustomerSubscriptionDeletedEvent,
    'customer.subscription.trial_will_end': ICustomerSubscriptionTrialWillEndEvent,
    'customer.discount.created': ICustomerDiscountCreatedEvent,
    'customer.discount.updated': ICustomerDiscountUpdatedEvent,
    'customer.discount.deleted': ICustomerDiscountDeletedEvent,
    'invoice.created': IInvoiceCreatedEvent,
    'invoice.updated': IInvoiceUpdatedEvent,
    'invoice.payment_succeeded': IInvoicePaymentSucceededEvent,
    'invoice.payment_failed': IInvoicePaymentFailedEvent,
    'invoiceitem.created': IInvoiceItemCreatedEvent,
    'invoiceitem.updated': IInvoiceItemUpdatedEvent,
    'invoiceitem.deleted': IInvoiceItemDeletedEvent,
    'plan.created': IPlanCreatedEvent,
    'plan.updated': IPlanUpdatedEvent,
    'plan.deleted': IPlanDeletedEvent,
    'coupon.created': ICouponCreatedEvent,
    'coupon.deleted': ICouponDeletedEvent,
    'transfer.created': ITransferCreatedEvent,
    'transfer.updated': ITransferUpdatedEvent,
    'transfer.paid': ITransferPaidEvent,
    'transfer.failed': ITransferFailedEvent,
    'ping': IPingEvent,
}

class StripeWebhooksView(grok.View):
    grok.name('stripe-webhooks')
    grok.require('zope2.Public')
    grok.context(IPloneSiteRoot)

    # These events will not be verified by an API callback
    unverified = ['ping',]

    def render(self):
        event_json = json.loads(self.request.get('BODY'))
        stripe_util = getUtility(IStripeUtility)

        mode = 'live'
        if event_json['livemode'] == False:
            mode = 'test'
        stripe_api = stripe_util.get_stripe_api(mode=mode)

        # Make sure we have a mapping for the event
        event_class = EVENTS_MAP[event_json['type']]

        # Fetch the event to verify authenticity, unless it is in the unverified list
        if event_json['type'] in self.unverified:
            data = event_json
        else:
            data = stripe_api.Event.retrieve(event_json['id'])

            # Verify that the id, type, and created date of the event match
            if data['id'] != event_json['id']:
                raise ValueError('Event id failed verification')
            if data['type'] != event_json['type']:
                raise ValueError('Event type failed verification')
            if data['created'] != event_json['created']:
                raise ValueError('Event creation date failed verification')

        # Send the event with data
        event = event_class(data)
        notify(event)

        return 'OK'



import transaction
import unittest2 as unittest
from zope.interface import Interface, implements
from zope.component import getSiteManager, getUtility
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.app.testing import SITE_OWNER_NAME, SITE_OWNER_PASSWORD
from plone.testing.z2 import Browser
from Products.Five.utilities.marker import mark
from collective.stripe.interfaces import IStripeModeChooser
from collective.stripe.utils import get_settings, IStripeUtility
from collective.stripe.testing import FUNCTIONAL_TESTING

class UtilsFunctionalTest(unittest.TestCase):
    # The layer's setup will run once before all of these tests run,
    # and its teardown will run once after all these tests run.
    layer = FUNCTIONAL_TESTING

    # setUp is run once before *each* of these tests.
    # This stuff can be moved to the layer's setupPloneSite if you want
    # it for all tests using the layer, not just this test class.
    def setUp(self):
        self.layer['portal'].validate_email = False
        portal = self.layer['portal']

        transaction.commit()
    
    # tearDown is run once after *each* of these tests.
    def tearDown(self):
        pass

    def test_global_utility(self):
        portal = self.layer['portal']
        util = getUtility(IStripeUtility)
        self.assertEqual(util.__class__.__name__, 'StripeUtility')

    def test_mode(self):
        portal = self.layer['portal']
        util = getUtility(IStripeUtility)
        settings = get_settings()

        settings.mode = 'test'
        mode = util.get_mode_for_context(portal)
        self.assertEqual(mode, 'test')

        settings.mode = 'live'
        mode = util.get_mode_for_context(portal)
        self.assertEqual(mode, 'live')

        # Now mark with IStripeModeChooser and test overriding the mode by context
        mark(portal, IStripeModeChooser)

        portal.stripe_mode = 'test'
        portal.get_stripe_mode = lambda: 'test'
        mode = util.get_mode_for_context(portal)
        self.assertEqual(mode, 'test')
        
        settings.mode = 'test'
        portal.get_stripe_mode = lambda: 'live'
        mode = util.get_mode_for_context(portal)
        self.assertEqual(mode, 'live')



import transaction
import unittest2 as unittest
from zope.component import getSiteManager
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.app.testing import SITE_OWNER_NAME, SITE_OWNER_PASSWORD
from plone.testing.z2 import Browser
from collective.stripe.utils import get_settings
from collective.stripe.testing import FUNCTIONAL_TESTING

class ControlPanelFunctionalTest(unittest.TestCase):
    # The layer's setup will run once before all of these tests run,
    # and its teardown will run once after all these tests run.
    layer = FUNCTIONAL_TESTING

    # setUp is run once before *each* of these tests.
    # This stuff can be moved to the layer's setupPloneSite if you want
    # it for all tests using the layer, not just this test class.
    def setUp(self):
        self.layer['portal'].validate_email = False
        portal = self.layer['portal']

        transaction.commit()
    
    # tearDown is run once after *each* of these tests.
    def tearDown(self):
        pass

    def test_controlpanel(self):
        portal = self.layer['portal']

        # Now as an admin user, go through the steps in the browser
        browser = Browser(portal)
        browser.handleErrors = False
        browser.addHeader('Authorization', 'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,))
        browser.open('http://nohost/plone/plone_control_panel')

        settings_link = browser.getLink('Stripe Payment Processing')
        settings_link.click()
        self.assertEqual('http://nohost/plone/@@stripe-settings', browser.url)
        
        # Set the values in the control panel
        browser.getControl(name='form.widgets.mode:list').value = ["test",]
        browser.getControl(name='form.widgets.live_secret_key').value = "1234567890"
        browser.getControl(name='form.widgets.live_publishable_key').value = "9876543210"
        browser.getControl(name='form.widgets.test_secret_key').value = "qwertyuiop"
        browser.getControl(name='form.widgets.test_publishable_key').value = "poiuytrewq"
        browser.getControl(name='form.buttons.save').click()

        # Check that the settings were changed in the registry
        settings = get_settings()
        self.assertEqual(settings.mode, "test")
        self.assertEqual(settings.live_secret_key, "1234567890")
        self.assertEqual(settings.live_publishable_key, "9876543210")
        self.assertEqual(settings.test_secret_key, "qwertyuiop")
        self.assertEqual(settings.test_publishable_key, "poiuytrewq")

